/*
 * vim:sw=2 ts=2 sts=2:et ai:bg=dark
 *
  mpbz2 - a program to do multi-threaded bzip2 compression using libbzip2
  
  Copyright Clint Byrum, 2004. All Rights Reserved.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  ---

  $Id: mpbz2.c 3 2004-08-23 22:11:51Z clint $
*/ 
#ifndef MPBZ2_H
#define MPBZ2_H

#include <pthread.h>
#include <stdio.h>
#include "ring_buffer.h"

#define BLOCK_SIZE 100000
#define DEFAULT_BUF_SIZE 8
#define DEFAULT_COMPRESS_LEVEL 5
#define DEFAULT_COMPRESS_THREADS 2

enum log_level {LOG_ERROR,LOG_INFO,LOG_DEBUG};

#ifndef __CPLUSPLUS
enum truth {false, true};
#endif

#ifndef DEBUGLEVEL
#define DEBUGLEVEL LOG_ERROR
#endif

typedef struct mpbz2_info_t {
  pthread_cond_t cond_compress;
  pthread_cond_t cond_read;
  pthread_cond_t cond_write;
  
  int read_threads;
  int compress_threads;
  int write_threads;

  mpbz2_ring_buffer* buf; 

#ifdef __CPLUSPLUS
  bool done;
  bool started_read;
  bool started_write;
  bool started_compress;

#else
  enum truth done;
  enum truth started_read;
  enum truth started_write;
  enum truth started_compress;

#endif
  
  FILE* outstream;
  FILE* instream;

  pthread_mutex_t lock;
} mpbz2_info;

#endif
