#include "ring_buffer.h"

mpbz2_buffer* mpbz2_create_buffer(int bufsize, int alloc_buffer) {
  mpbz2_buffer* buf;
  buf = (mpbz2_buffer*)malloc(sizeof(mpbz2_buffer));

  if(alloc_buffer) {
    buf->data = malloc(bufsize);
    buf->size = bufsize;
  } else {
    buf->data = NULL;
    buf->size = 0;
  }
  buf->len = 0;

  return buf;
}

void mpbz2_destroy_buffer(mpbz2_buffer* buf) {
  if(!buf) {
    return;
  }
  if(buf->data) {
    free(buf->data);
  }
  free(buf);
}

mpbz2_queue_item* mpbz2_create_queue_item(int bufsize) {
  mpbz2_queue_item* item;
  item = (mpbz2_queue_item*)malloc(sizeof(mpbz2_queue_item));
  
  item->next = NULL;
  item->input = mpbz2_create_buffer(bufsize, true);
  item->output = NULL;
  item->state = BLOCK_EMPTY;
  item->serial = -1;

  return item;
}

void mpbz2_destroy_queue_item(mpbz2_queue_item* item) {
  mpbz2_destroy_buffer(item->input);
  mpbz2_destroy_buffer(item->output);
  free(item);
}

mpbz2_ring_buffer* mpbz2_create_ring_buffer(int size, int bufsize) {
  mpbz2_ring_buffer* buffer;
  mpbz2_queue_item* item;
  int i;
  
  buffer = (mpbz2_ring_buffer*)malloc(sizeof(mpbz2_ring_buffer));
  item = mpbz2_create_queue_item(bufsize);
  buffer->next_empty = buffer->next_full = buffer->next_compressed = item;
  for(i = size - 1; i; i--) {
    item = item->next = mpbz2_create_queue_item(bufsize);
  }
  item->next = buffer->next_empty;
  
  buffer->num_empty = size;
  buffer->num_compressed = buffer->num_full = 0;

  pthread_mutex_init(&buffer->lock, NULL);

  return buffer;
}

void mpbz2_destroy_ring_buffer(mpbz2_ring_buffer* buf) {
  mpbz2_queue_item* item;
  mpbz2_queue_item* next;

  item = buf->next_empty->next;
  mpbz2_destroy_queue_item(buf->next_empty);
  while(item != buf->next_empty) {
    next = item->next;
    mpbz2_destroy_queue_item(item);
    item = next;
  }

  pthread_mutex_destroy(&buf->lock);

  free(buf);
}
