#ifndef MPBZ2_RING_BUFFER_H
#define MPBZ2_RING_BUFFER_H

#include <stdlib.h>
#include <pthread.h>

enum queue_item_state { BLOCK_EMPTY, BLOCK_FULL, BLOCK_COMPRESSED };

typedef struct mpbz2_buffer_t {
  void *data;
  size_t len;
  size_t size;
} mpbz2_buffer;

typedef struct mpbz2_queue_item_t {
  mpbz2_buffer* input;
  mpbz2_buffer* output;
  enum queue_item_state state; 
  size_t serial;

  struct mpbz2_queue_item_t *next;
} mpbz2_queue_item;

/* All of these pointers point to the same list */
typedef struct mpbz2_ring_buffer_t {
  mpbz2_queue_item* next_empty;
  mpbz2_queue_item* next_full;
  mpbz2_queue_item* next_compressed;

  int num_empty;
  int num_full;
  int num_compressed;

  /* Lock this if you are manipulating any pointers */
  pthread_mutex_t lock;
} mpbz2_ring_buffer;

mpbz2_buffer* mpbz2_create_buffer(int bufsize, int alloc_buffer);
void mpbz2_destroy_buffer(mpbz2_buffer* buf);
mpbz2_queue_item* mpbz2_create_queue_item(int bufsize);
void mpbz2_destroy_queue_item(mpbz2_queue_item* item);
mpbz2_ring_buffer* mpbz2_create_ring_buffer(int size, int bufsize);
void mpbz2_destroy_ring_buffer(mpbz2_ring_buffer* buf);

#include "mpbz2.h"

#endif
