#include "thread_funcs.h"

void * readController(void *idp) {
  mpbz2_info* inf = (mpbz2_info*)idp;
  mpbz2_ring_buffer* buf = inf->buf;
  size_t totread = 0;
  mpbz2_queue_item *q;

  pthread_mutex_lock(&inf->lock);
  inf->read_threads++;
  inf->started_read = true;
  pthread_mutex_unlock(&inf->lock);
  
  do {
    //logMsg(LOG_INFO, "readController is ready...\n");
    pthread_mutex_lock(&buf->lock);
    //logMsg(LOG_INFO, "readController got lock...\n");
    while(buf->num_empty == 0) {
      //logMsg(LOG_INFO, "readController is waiting...\n");
      pthread_cond_wait(&inf->cond_read, &buf->lock);
    }
    q = buf->next_empty;
    pthread_mutex_unlock(&buf->lock);

    q->input->len = fread(q->input->data, 1, q->input->size, inf->instream);
    totread += q->input->len;

    if(q->input->len == 0) {
      break;
    }

    //logMsg(LOG_INFO, "readController read a block!\n");
    logMsg(LOG_INFO, "Read %d bytes, %d byte buffer\n", q->input->len, q->input->size);
    
    pthread_mutex_lock(&buf->lock);
    buf->num_empty--;
    buf->num_full++;
    q->state = BLOCK_FULL;
    buf->next_empty = buf->next_empty->next;
    pthread_mutex_unlock(&buf->lock);

    pthread_cond_broadcast(&inf->cond_compress);
  } while((!inf->started_compress || inf->compress_threads > 0) && q->input->len == q->input->size);
  logMsg(LOG_DEBUG,"EOF Reached\n");

  inf->done = true;

  /* Wake up any remaining compress threads, make sure they don't deadlock */
  pthread_cond_broadcast(&inf->cond_compress);

  pthread_mutex_lock(&inf->lock);
  inf->read_threads--;
  pthread_mutex_unlock(&inf->lock);
  
  logMsg(LOG_INFO,"readController exiting.\n");

  return NULL;
}

void * compressController(void *idp) {
  mpbz2_info* inf = (mpbz2_info*)idp;
  mpbz2_ring_buffer* buf = inf->buf;
  pthread_mutex_lock(&inf->lock);
  inf->compress_threads++;
  inf->started_compress = true;
  while(!inf->started_read) {
    pthread_cond_wait(&inf->cond_compress, &inf->lock);
  }
  pthread_mutex_unlock(&inf->lock);
  do {
    mpbz2_queue_item* q;
    //logMsg(LOG_INFO, "compressController is ready...\n");
    pthread_mutex_lock(&buf->lock);
    //logMsg(LOG_INFO, "compressController got lock...\n");
    while(buf->num_full == 0 && inf->read_threads > 0) {
      //logMsg(LOG_INFO, "compressController is waiting...\n");
      pthread_cond_wait(&inf->cond_compress, &buf->lock);
    }
    q = buf->next_full;
    if(q->state != BLOCK_FULL) {
      pthread_mutex_unlock(&buf->lock);
      break;
    }
    buf->next_full = buf->next_full->next;
    buf->num_full--;
    pthread_mutex_unlock(&buf->lock);

    //logMsg(LOG_INFO, "compressController grabbed a block!\n");

    compressBlock(q);
    
    pthread_mutex_lock(&buf->lock);
    buf->num_compressed++;
    q->state = BLOCK_COMPRESSED;
    pthread_mutex_unlock(&buf->lock);

    logMsg(LOG_INFO, "Signalling writer...\n");
    pthread_cond_signal(&inf->cond_write);
  } while(buf->num_full > 0 || inf->read_threads > 0);

  /* Wake up remaining compress threads so they can exit */
  //logMsg(LOG_INFO, "compressController starting exit routine...\n");
  pthread_mutex_lock(&inf->lock);
  inf->compress_threads--;
  pthread_mutex_unlock(&inf->lock);

  //logMsg(LOG_INFO, "compressController signalling...\n");
  pthread_cond_signal(&inf->cond_compress);
  pthread_cond_signal(&inf->cond_write);
  logMsg(LOG_INFO,"compressController exiting... \n");
  return NULL;
  //  pthread_exit(0);
}

void * writeController(void *idp) {
  mpbz2_info* inf = (mpbz2_info*)idp;
  mpbz2_ring_buffer* buf = inf->buf;
  pthread_mutex_lock(&inf->lock);
  inf->write_threads++;
  inf->started_write = true;
  while(!inf->started_compress) {
    pthread_cond_wait(&inf->cond_write, &inf->lock);
  }
  pthread_mutex_unlock(&inf->lock);
  do {
    mpbz2_queue_item* q;
    size_t written;
    //logMsg(LOG_INFO, "writeController is ready...\n");
    pthread_mutex_lock(&buf->lock);
    //logMsg(LOG_INFO, "writeController got lock...\n");
    while(buf->next_compressed->state != BLOCK_COMPRESSED && inf->compress_threads > 0) {
      //logMsg(LOG_INFO, "writeController is waiting...\n");
      pthread_cond_wait(&inf->cond_write, &buf->lock);
    }
    q = buf->next_compressed;
    if(q->state != BLOCK_COMPRESSED) {
      logMsg(LOG_INFO, "Exiting because next block was not compressed\n");
      pthread_mutex_unlock(&buf->lock);
      break;
    }
    buf->num_compressed--;
    buf->next_compressed = buf->next_compressed->next;
    pthread_mutex_unlock(&buf->lock);

    written = fullWrite(inf->outstream, q->output->data, q->output->len);
    if(written != q->output->len) {
      logMsg(LOG_ERROR, "Could not write more data to disk!\n");
      exit(1);
    }
    mpbz2_destroy_buffer(q->output);
    q->output = NULL;
    
    pthread_mutex_lock(&buf->lock);
    buf->num_empty++;
    q->state = BLOCK_EMPTY;
    pthread_mutex_unlock(&buf->lock);
    pthread_cond_broadcast(&inf->cond_read);

    logMsg(LOG_INFO, "writeController wrote a block!\n");

  } while(buf->num_compressed > 0 || inf->compress_threads > 0);

  pthread_mutex_lock(&inf->lock);
  inf->write_threads--;
  pthread_mutex_unlock(&inf->lock);

  logMsg(LOG_INFO, "next_empty: %x, next_full: %x, next_compressed: %x\n", buf->next_empty, buf->next_full, buf->next_compressed);

  logMsg(LOG_INFO,"writeController exiting... \n");
  //pthread_exit(0);
  return NULL;
}
